//
// Copyright (C) 2013-2016  Taiyou Sakashitai, Hiroko Yamada
//
// Calculate Maximum and Minimum 2015/08/10
// Modified for using Julidan day 2016/07/01
//

import java.io.*;
import java.util.*;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.Calendar;

public class maxmin
{

// Length of the Data in Column/Line Direction 
static int X;
static int Y;

// Year/Month/Hour/Minute
static int YYYY;
static int MM;
static int HH;
static int MN;

// UTC Year/Month/Day/Hour
static int UYYYY;
static int UMM;
static int UDD;
static int UHH;

// Number of Used File
static int nFile = 0;

// Maximum/Minimum
static float MAX[][];
static float MIN[][];

// Calculation Start Day
static int Start_DD;

// Calculation End Day
static int End_DD;

// File Type
static String ftype;

// Directory Name to Read Original Data
static String input_DIR;

// Directory Name to Write Maximum/Minimum Data
static String MAXMIN_DIR;

// TIME DIFF 
static double TIME_DIFF;
static String TIME_Code;

// Julian day UTC 
static double jd_UTC = 0;


public static void main(String args[])
{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );


  // Filename of Maximum Data
  String wfilename_max;

  // Filename of Minimum Data
  String wfilename_min;
  
  // Year/Month/hour/Minute
  YYYY = Integer.valueOf(args[0]).intValue();	
  MM   = Integer.valueOf(args[1]).intValue();
  HH   = Integer.valueOf(args[2]).intValue();
  MN   = Integer.valueOf(args[3]).intValue();

 // output/YYYYMM Dir confirm and create
  File newfile = new File("output/" + YYYY + df.format(MM));
   	if (newfile.exists()) {
		System.out.println( newfile + " already exists");
	}else if (newfile.mkdir()){
    		System.out.println("Succeeded to create a directory");
  	}else{
    		System.out.println("Failed to create a directory");
  	}

  date_errorcheck();
  initialization();

  // Calculate Maximum/Minimum
  cal_maxmin(input_DIR);

  String ftype_cut = ftype.substring(0, ftype.length()-4);

  wfilename_max = MAXMIN_DIR + YYYY + df.format(MM) + "/" + YYYY + df.format(MM) + "_" + df.format(HH) + df.format(MN) +"." + TIME_Code + ftype_cut + ".max.bin";
  wfilename_min = MAXMIN_DIR + YYYY + df.format(MM) + "/" + YYYY + df.format(MM) + "_" + df.format(HH) + df.format(MN) +"." + TIME_Code +  ftype_cut + ".min.bin";

  bin bb = new bin();

  // Write Maximum/Minimum Data
  if(nFile > 0)
  {
     bb.f2bin(wfilename_max, X, Y, MAX);
     bb.f2bin(wfilename_min, X, Y, MIN);

  }else{

     System.out.println("Data of the specified time does not exist");

  }//fi


  return;


}//main


public static void cal_maxmin(String DIR)
{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );


  int I;

  // Data Read
  float DR[][] = new float[Y+1][X+1];

  // Filename of Original Data
  String filename;

  File file;
  
  // UTC Time
  int UTime[] = new int[5+1];
  jday jday_jul = new jday();

  bin bb = new bin();

  // Calculate Maximum/Minimum
  for(I=Start_DD;I<=End_DD;I++)
  {
  	// Local Time  to JD(Local Time)　　ex. 2014.05.02.12(JST)
	double jd_jul_ymdhn=(double)jday_jul.TransDateToJD(YYYY,MM,I,HH,MN,0);
  	// Seek JD(UTC) 
	jd_UTC= jd_jul_ymdhn - (double)TIME_DIFF/24;
  	// JD(UTC)  to Calendar UYYY,UMM,UDD,UHH
	UTime = jday_jul.jd2ymdhn(jd_UTC);
 
	 UYYYY = UTime[1];
     UMM = UTime[2];
     UDD = UTime[3];
     UHH = UTime[4];

	 filename = input_DIR + UYYYY + df.format(UMM) + "/" + UYYYY + df.format(UMM) + df.format(UDD) + df.format(UHH) + df.format(MN) + ftype;

     file = new File(filename);

     if(file.exists())
     {
        DR = bb.bin2f(filename, X, Y);
        check_maxmin(DR);
        nFile++;

     }//fi
  }//I


  return;


}//cal_maxmin


public static void check_maxmin(float DR[][])
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        // Max/Min Check
        if(DR[I][J] > MAX[I][J]) MAX[I][J] = DR[I][J];
        if(DR[I][J] < MIN[I][J]) MIN[I][J] = DR[I][J];

     }//J
  }//I


  return;


}//check_maxmin


public static void date_errorcheck()
{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );


  // Last Day of the Month
  int Last_DD;


  // Year Check
  if(YYYY <= 0)
  {
     System.out.println("Year " + YYYY + " is less than or equal to 0");
     System.exit(1);

  }//fi

  // Month Check
  if(MM <= 0 || MM > 12)
  {
     System.out.println("Month " + MM + " does not exist");
     System.exit(1);

  }//fi

  // Hour Check
  if(HH < 0 || HH >= 24)
  {
     System.out.println("Hour " + HH + " does not exist");
     System.exit(1);

  }//fi

  // Minute Check
  if(MN < 0 || MN >= 60)
  {
     System.out.println("Minute " + MN + " does not exist");
     System.exit(1);

  }//fi


  return;


}//date_errorcheck


public static void initialization()
{

  int I,J;


  read_formatfile();


  MAX = new float[Y+1][X+1];
  MIN = new float[Y+1][X+1];


  // MIN Initialization
  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        MIN[I][J] = 10000;

     }//J
  }//I


  return;


}//initialization


static public void read_formatfile()
{

  // Present Line
  int pline=1;

  String filename;
  String readline;
  String param[] = new String [100+1]; 


  filename = "config/format.txt";


  // Read Format File
  try
  {
       
    System.out.print(filename +"  Loading ...");

    BufferedReader br = new BufferedReader(new FileReader(filename));

    while( (readline = br.readLine()) != null ){

        StringTokenizer st = new StringTokenizer(readline);

	 param[pline] =  st.nextToken(", ");

        pline++;
                 
    }//while

    br.close();
	
  }catch (IOException e){

      System.err.println("Initialize error\n");
      System.exit(1);

   }//try

  System.out.println(" finish");
  

  TIME_DIFF = Double.valueOf(param[1]).doubleValue();
  TIME_Code = param[2];

  X = Integer.valueOf(param[3]).intValue();
  Y = Integer.valueOf(param[4]).intValue();

  ftype = param[5];

  input_DIR = param[6];
  MAXMIN_DIR = param[7];

  Start_DD = Integer.valueOf(param[11]).intValue();
  End_DD = Integer.valueOf(param[12]).intValue();


  return;


}//read_formatfile



}//EOF

